"""
WorkflowStatus class module.
"""
from __future__ import annotations

from digitalhub_core.entities._base.status import Status


class WorkflowStatus(Status):
    """
    Status class for workflow entities.
    """
