"""
FunctionStatus class module.
"""
from __future__ import annotations
from digitalhub_core.entities._base.status import Status


class FunctionStatus(Status):
    """
    Status class for function entities.
    """
