"""
DataitemStatus class module.
"""
from __future__ import annotations

from digitalhub_core.entities._base.status import Status


class DataitemStatus(Status):
    """
    Status class for dataitem entities.
    """
