"""
Common constants used in the SDK.
"""

####################
# Entities
####################

PROJ = "projects"
FUNC = "functions"
WKFL = "workflows"
TASK = "tasks"
RUNS = "runs"
LOGS = "logs"
ARTF = "artifacts"
DTIT = "dataitems"
MDLS = "models"
